@@ -1,7 +1,7 @@
 /*
  * The XML and API file generator module for SIP.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -57,8 +57,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
 static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp);
 static void xmlIndent(int indent, FILE *fp);
 static const char *dirAttribute(argDef *ad);
-static void exportDefaultValue(argDef *ad, int in_str, FILE *fp);
 static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope);
+static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
+        int sec, int names, int defaults, int in_str, int is_signal);
 
 
 /*
@@ -239,8 +240,8 @@ static int apiOverload(sipSpec *pt, moduleDef *mod, classDef *scope,
     prScopedPythonName(fp, scope, od->common->pyname->text);
     fprintf(fp, "?%d", METHOD_ID);
 
-    need_sec = prPythonSignature(pt, fp, &od->pysig, sec, TRUE, TRUE, FALSE,
-            FALSE);
+    need_sec = exportPythonSignature(pt, fp, &od->pysig, sec, TRUE, TRUE,
+            FALSE, FALSE);
 
     fprintf(fp, "\n");
 
@@ -281,9 +282,7 @@ static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
             fprintf(fp, " %s", ad->name->text);
 
         fprintf(fp, "=");
-        prcode(fp, "%M");
-        exportDefaultValue(ad, in_str, fp);
-        prcode(fp, "%M");
+        prDefaultValue(ad, in_str, fp);
     }
 
     return TRUE;
@@ -612,6 +611,11 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
         fprintf(fp, "\"");
     }
 
+    if (isVirtual(od))
+    {
+        fprintf(fp, " virtual=\"1\"");
+    }
+
     if (xtnds != NULL)
     {
         fprintf(fp, " extends=\"");
@@ -723,9 +727,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
      */
     if (ad->defval && (dir == NULL || strcmp(dir, "out") != 0))
     {
-        prcode(fp, " default=\"%M");
-        exportDefaultValue(ad, FALSE, fp);
-        prcode(fp, "%M\"");
+        prcode(fp, " default=\"");
+        prDefaultValue(ad, FALSE, fp);
+        prcode(fp, "\"");
     }
 
     fprintf(fp, "/>\n");
@@ -786,6 +790,10 @@ static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp)
     case mapped_type:
         type_type = "mappedtype";
         break;
+
+    /* Suppress a compiler warning. */
+    default:
+        ;
     }
 
     if ((type_name = pyType(pt, ad, sec, &type_scope)) != NULL)
@@ -811,38 +819,6 @@ static void xmlIndent(int indent, FILE *fp)
 }
 
 
-/*
- * Export the default value of an argument.
- */
-static void exportDefaultValue(argDef *ad, int in_str, FILE *fp)
-{
-    /* Use any explicitly provided documentation. */
-    if (ad->docval != NULL)
-    {
-        prcode(fp, "%s", ad->docval);
-        return;
-    }
-
-    /* Translate some special cases. */
-    if (ad->defval->next == NULL && ad->defval->vtype == numeric_value)
-    {
-        if (ad->nrderefs > 0 && ad->defval->u.vnum == 0)
-        {
-            prcode(fp, "None");
-            return;
-        }
-
-        if (ad->atype == bool_type || ad->atype == cbool_type)
-        {
-            prcode(fp, ad->defval->u.vnum ? "True" : "False");
-            return;
-        }
-    }
-
-    generateExpression(ad->defval, in_str, fp);
-}
-
-
 /*
  * Get the Python representation of a type.
  */
@@ -987,13 +963,17 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
         break;
 
     case ustring_type:
+        /* Correct for Python v3. */
+        type_name = "bytes";
+        break;
+
     case string_type:
     case sstring_type:
     case wstring_type:
     case ascii_string_type:
     case latin1_string_type:
     case utf8_string_type:
-        type_name = "str";
+        type_name = isArray(ad) ? "bytes" : "str";
         break;
 
     case byte_type:
@@ -1008,6 +988,7 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
     case short_type:
     case int_type:
     case cint_type:
+    case ssize_type:
         type_name = "int";
         break;
 
@@ -1072,27 +1053,11 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
 }
 
 
-/*
- * Generate a scoped Python name.
- */
-void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname)
-{
-    if (scope != NULL)
-    {
-        prScopedPythonName(fp, scope->ecd, NULL);
-        fprintf(fp, "%s.", scope->pyname->text);
-    }
-
-    if (pyname != NULL)
-        fprintf(fp, "%s", pyname);
-}
-
-
 /*
  * Generate a Python signature.
  */
-int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str, int is_signal)
+static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
+        int sec, int names, int defaults, int in_str, int is_signal)
 {
     int need_sec = FALSE, need_comma = FALSE, is_res, nr_out, a;
 
