@@ -1,7 +1,7 @@
 /*
  * The main module for SIP.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2017 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -37,11 +37,12 @@ stringList *includeDirList;
 
 static char *sipPackage = PACKAGE;
 static int warnings = FALSE;
+static int warnings_are_fatal = FALSE;
 
 
-static void help(void);
-static void version(void);
-static void usage(void);
+SIP_NORETURN static void help(void);
+SIP_NORETURN static void version(void);
+SIP_NORETURN static void usage(void);
 static char parseopt(int,char **,char *,char **,int *,char **);
 static int parseInt(char *,char);
 
@@ -49,9 +50,9 @@ static int parseInt(char *,char);
 int main(int argc, char **argv)
 {
     char *filename, *docFile, *codeDir, *srcSuffix, *flagFile, *consModule;
-    char arg, *optarg, *buildFile, *apiFile, *xmlFile;
+    char arg, *optarg, *buildFile, *apiFile, *xmlFile, *pyiFile;
     int optnr, exceptions, tracing, releaseGIL, parts, protHack, docs;
-    int timestamp;
+    int timestamp, was_flagFile, py_debug;
     KwArgs kwArgs;
     FILE *file;
     sipSpec spec;
@@ -68,8 +69,10 @@ int main(int argc, char **argv)
     docFile = NULL;
     srcSuffix = NULL;
     flagFile = NULL;
+    was_flagFile = FALSE;
     apiFile = NULL;
     xmlFile = NULL;
+    pyiFile = NULL;
     consModule = NULL;
     extracts = NULL;
     exceptions = FALSE;
@@ -80,11 +83,12 @@ int main(int argc, char **argv)
     protHack = FALSE;
     docs = FALSE;
     timestamp = TRUE;
+    py_debug = FALSE;
 
     /* Parse the command line. */
     optnr = 1;
 
-    while ((arg = parseopt(argc, argv, "hVa:b:B:ec:d:gI:j:km:op:Prs:t:Twx:X:z:", &flagFile, &optnr, &optarg)) != '\0')
+    while ((arg = parseopt(argc, argv, "hVa:b:B:ec:d:DfgI:j:km:op:Prs:t:Twx:X:y:z:", &flagFile, &optnr, &optarg)) != '\0')
         switch (arg)
         {
         case 'o':
@@ -112,6 +116,11 @@ int main(int argc, char **argv)
             xmlFile = optarg;
             break;
 
+        case 'y':
+            /* Where to generate the .pyi file. */
+            pyiFile = optarg;
+            break;
+
         case 'b':
             /* Generate a build file. */
             buildFile = optarg;
@@ -143,6 +152,7 @@ int main(int argc, char **argv)
                 fatal("The -z flag cannot be specified in an argument file\n");
 
             flagFile = optarg;
+            was_flagFile = TRUE;
             break;
 
         case 'c':
@@ -155,13 +165,21 @@ int main(int argc, char **argv)
             docFile = optarg;
             break;
 
+        case 'D':
+            /* Generate code for a debug build of Python. */
+            py_debug = TRUE;
+            break;
+
         case 't':
             /* Which platform or version to generate code for. */
             appendString(&versions,optarg);
             break;
 
         case 'T':
-            /* Disable the timestamp in the header of generated files. */
+            /*
+             * Disable the timestamp in the header of generated files.  It is
+             * now ignored apart from triggering a deprecation warning.
+             */
             timestamp = FALSE;
             break;
 
@@ -195,6 +213,11 @@ int main(int argc, char **argv)
             warnings = TRUE;
             break;
 
+        case 'f':
+            /* Warning messages are fatal. */
+            warnings_are_fatal = TRUE;
+            break;
+
         case 'k':
             /* Allow keyword arguments in functions and methods. */
             kwArgs = AllKwArgs;
@@ -235,6 +258,12 @@ int main(int argc, char **argv)
     if (kwArgs != NoKwArgs)
         warning(DeprecationWarning, "the -k flag is deprecated\n");
 
+    if (!timestamp)
+        warning(DeprecationWarning, "the -T flag is ignored and deprecated\n");
+
+    if (was_flagFile)
+        warning(DeprecationWarning, "the -z flag is deprecated\n");
+
     /* Parse the input file. */
     parse(&spec, file, filename, versions, backstops, xfeatures, kwArgs,
             protHack);
@@ -245,7 +274,7 @@ int main(int argc, char **argv)
     /* Generate code. */
     generateCode(&spec, codeDir, buildFile, docFile, srcSuffix, exceptions,
             tracing, releaseGIL, parts, versions, xfeatures, consModule, docs,
-            timestamp);
+            py_debug);
 
     /* Generate any extracts. */
     generateExtracts(&spec, extracts);
@@ -258,6 +287,10 @@ int main(int argc, char **argv)
     if (xmlFile != NULL)
         generateXML(&spec, spec.module,  xmlFile);
 
+    /* Generate the .pyi file. */
+    if (pyiFile != NULL)
+        generateTypeHints(&spec, spec.module,  pyiFile);
+
     /* All done. */
     return 0;
 }
@@ -274,10 +307,19 @@ static char parseopt(int argc, char **argv, char *opts, char **flags,
     int optnr;
     static FILE *fp = NULL;
 
+    optnr = *optnrp;
+
     /* Deal with any file first. */
 
     fname = *flags;
 
+    /* Support the sip5 method of passing arguments in a file. */
+    if (fname == NULL && optnr < argc && argv[optnr][0] == '@')
+    {
+        fname = *flags = &argv[optnr][1];
+        *optnrp = ++optnr;
+    }
+
     if (fname != NULL && fp == NULL && (fp = fopen(fname,"r")) == NULL)
         fatal("Unable to open %s\n",fname);
 
@@ -354,8 +396,6 @@ static char parseopt(int argc, char **argv, char *opts, char **flags,
 
     /* Check there is an argument and it is a switch. */
 
-    optnr = *optnrp;
-
     if (optnr >= argc || argv[optnr] == NULL || argv[optnr][0] != '-')
         return '\0';
 
@@ -474,32 +514,44 @@ void warning(Warning w, const char *fmt, ...)
     va_end(ap);
 
     if (strchr(fmt, '\n') != NULL)
+    {
+        if (warnings_are_fatal)
+            exit(1);
+
         start = TRUE;
+    }
 }
 
 
 /*
  * Display all or part of a one line error message describing a fatal error.
- * If the message is complete (it has a newline) then the program exits.
  */
-void fatal(char *fmt,...)
+void fatal(const char *fmt, ...)
 {
-    static int start = TRUE;
-
     va_list ap;
 
-    if (start)
-    {
-        fprintf(stderr,"%s: ",sipPackage);
-        start = FALSE;
-    }
+    fatalStart();
 
     va_start(ap,fmt);
     vfprintf(stderr,fmt,ap);
     va_end(ap);
 
-    if (strchr(fmt,'\n') != NULL)
-        exit(1);
+    exit(1);
+}
+
+
+/*
+ * Make sure the start of a fatal message is handled.
+ */
+void fatalStart()
+{
+    static int start = TRUE;
+
+    if (start)
+    {
+        fprintf(stderr, "%s: ", sipPackage);
+        start = FALSE;
+    }
 }
 
 
@@ -520,7 +572,7 @@ static void help(void)
 {
     printf(
 "Usage:\n"
-"    %s [-h] [-V] [-a file] [-b file] [-B tag] [-c dir] [-d file] [-e] [-g] [-I dir] [-j #] [-k] [-m file] [-o] [-p module] [-P] [-r] [-s suffix] [-t tag] [-T] [-w] [-x feature] [-X id:file] [-z file] [file]\n"
+"    %s [-h] [-V] [-a file] [-b file] [-B tag] [-c dir] [-d file] [-D] [-e] [-f] [-g] [-I dir] [-j #] [-k] [-m file] [-o] [-p module] [-P] [-r] [-s suffix] [-t tag] [-T] [-w] [-x feature] [-X id:file] [-z file] [@file] [file]\n"
 "where:\n"
 "    -h          display this help message\n"
 "    -V          display the %s version number\n"
@@ -529,7 +581,9 @@ static void help(void)
 "    -B tag      add tag to the list of timeline backstops\n"
 "    -c dir      the name of the code directory [default not generated]\n"
 "    -d file     the name of the documentation file (deprecated) [default not generated]\n"
+"    -D          generate code for a debug build of Python\n"
 "    -e          enable support for exceptions [default disabled]\n"
+"    -f          warnings are handled as errors\n"
 "    -g          always release and reacquire the GIL [default only when specified]\n"
 "    -I dir      look in this directory when including files\n"
 "    -j #        split the generated code into # files [default 1 per class]\n"
@@ -541,11 +595,12 @@ static void help(void)
 "    -r          generate code with tracing enabled [default disabled]\n"
 "    -s suffix   the suffix to use for C or C++ source files [default \".c\" or \".cpp\"]\n"
 "    -t tag      the version/platform to generate code for\n"
-"    -T          disable the timestamp in the header of generated files\n"
 "    -w          enable warning messages\n"
 "    -x feature  this feature is disabled\n"
 "    -X id:file  create the extracts for an id in a file\n"
+"    -y file     the name of the .pyi stub file [default not generated]\n"
 "    -z file     the name of a file containing more command line flags\n"
+"    @file       the name of a file containing more command line flags\n"
 "    file        the name of the specification file [default stdin]\n"
         , sipPackage, sipPackage);
 
@@ -558,5 +613,9 @@ static void help(void)
  */
 static void usage(void)
 {
-    fatal("Usage: %s [-h] [-V] [-a file] [-b file] [-B tag] [-c dir] [-d file] [-e] [-g] [-I dir] [-j #] [-k] [-m file] [-o] [-p module] [-P] [-r] [-s suffix] [-t tag] [-T] [-w] [-x feature] [-X id:file] [-z file] [file]\n", sipPackage);
+    fatal("Usage: %s [-h] [-V] [-a file] [-b file] [-B tag] [-c dir] "
+            "[-d file] [-D] [-e] [-f] [-g] [-I dir] [-j #] [-k] [-m file] [-o] "
+            "[-p module] [-P] [-r] [-s suffix] [-t tag] [-w] [-x feature] "
+            "[-X id:file] [-y file] [-z file] [@file] [file]\n",
+            sipPackage);
 }
