@@ -84,105 +84,109 @@
      TK_PICKLECODE = 300,
      TK_VIRTUALCALLCODE = 301,
      TK_METHODCODE = 302,
-     TK_INSTANCECODE = 303,
-     TK_FROMTYPE = 304,
-     TK_TOTYPE = 305,
-     TK_TOSUBCLASS = 306,
-     TK_INCLUDE = 307,
-     TK_OPTINCLUDE = 308,
-     TK_IMPORT = 309,
-     TK_EXPHEADERCODE = 310,
-     TK_MODHEADERCODE = 311,
-     TK_TYPEHEADERCODE = 312,
-     TK_MODULE = 313,
-     TK_CMODULE = 314,
-     TK_CONSMODULE = 315,
-     TK_COMPOMODULE = 316,
-     TK_CLASS = 317,
-     TK_STRUCT = 318,
-     TK_PUBLIC = 319,
-     TK_PROTECTED = 320,
-     TK_PRIVATE = 321,
-     TK_SIGNALS = 322,
-     TK_SIGNAL_METHOD = 323,
-     TK_SLOTS = 324,
-     TK_SLOT_METHOD = 325,
-     TK_BOOL = 326,
-     TK_SHORT = 327,
-     TK_INT = 328,
-     TK_LONG = 329,
-     TK_FLOAT = 330,
-     TK_DOUBLE = 331,
-     TK_CHAR = 332,
-     TK_WCHAR_T = 333,
-     TK_VOID = 334,
-     TK_PYOBJECT = 335,
-     TK_PYTUPLE = 336,
-     TK_PYLIST = 337,
-     TK_PYDICT = 338,
-     TK_PYCALLABLE = 339,
-     TK_PYSLICE = 340,
-     TK_PYTYPE = 341,
-     TK_PYBUFFER = 342,
-     TK_VIRTUAL = 343,
-     TK_ENUM = 344,
-     TK_SIGNED = 345,
-     TK_UNSIGNED = 346,
-     TK_SCOPE = 347,
-     TK_LOGICAL_OR = 348,
-     TK_CONST = 349,
-     TK_STATIC = 350,
-     TK_SIPSIGNAL = 351,
-     TK_SIPSLOT = 352,
-     TK_SIPANYSLOT = 353,
-     TK_SIPRXCON = 354,
-     TK_SIPRXDIS = 355,
-     TK_SIPSLOTCON = 356,
-     TK_SIPSLOTDIS = 357,
-     TK_SIPSSIZET = 358,
-     TK_NUMBER_VALUE = 359,
-     TK_REAL_VALUE = 360,
-     TK_TYPEDEF = 361,
-     TK_NAMESPACE = 362,
-     TK_TIMELINE = 363,
-     TK_PLATFORMS = 364,
-     TK_FEATURE = 365,
-     TK_LICENSE = 366,
-     TK_QCHAR_VALUE = 367,
-     TK_TRUE_VALUE = 368,
-     TK_FALSE_VALUE = 369,
-     TK_NULL_VALUE = 370,
-     TK_OPERATOR = 371,
-     TK_THROW = 372,
-     TK_QOBJECT = 373,
-     TK_EXCEPTION = 374,
-     TK_RAISECODE = 375,
-     TK_VIRTERRORCODE = 376,
-     TK_EXPLICIT = 377,
-     TK_TEMPLATE = 378,
-     TK_ELLIPSIS = 379,
-     TK_DEFMETATYPE = 380,
-     TK_DEFSUPERTYPE = 381,
-     TK_PROPERTY = 382,
-     TK_FORMAT = 383,
-     TK_GET = 384,
-     TK_ID = 385,
-     TK_KWARGS = 386,
-     TK_LANGUAGE = 387,
-     TK_LICENSEE = 388,
-     TK_NAME = 389,
-     TK_OPTIONAL = 390,
-     TK_ORDER = 391,
-     TK_REMOVELEADING = 392,
-     TK_SET = 393,
-     TK_SIGNATURE = 394,
-     TK_TIMESTAMP = 395,
-     TK_TYPE = 396,
-     TK_USEARGNAMES = 397,
-     TK_ALLRAISEPYEXC = 398,
-     TK_CALLSUPERINIT = 399,
-     TK_DEFERRORHANDLER = 400,
-     TK_VERSION = 401
+     TK_PREMETHODCODE = 303,
+     TK_INSTANCECODE = 304,
+     TK_FROMTYPE = 305,
+     TK_TOTYPE = 306,
+     TK_TOSUBCLASS = 307,
+     TK_INCLUDE = 308,
+     TK_OPTINCLUDE = 309,
+     TK_IMPORT = 310,
+     TK_EXPHEADERCODE = 311,
+     TK_MODHEADERCODE = 312,
+     TK_TYPEHEADERCODE = 313,
+     TK_MODULE = 314,
+     TK_CMODULE = 315,
+     TK_CONSMODULE = 316,
+     TK_COMPOMODULE = 317,
+     TK_CLASS = 318,
+     TK_STRUCT = 319,
+     TK_PUBLIC = 320,
+     TK_PROTECTED = 321,
+     TK_PRIVATE = 322,
+     TK_SIGNALS = 323,
+     TK_SIGNAL_METHOD = 324,
+     TK_SLOTS = 325,
+     TK_SLOT_METHOD = 326,
+     TK_BOOL = 327,
+     TK_SHORT = 328,
+     TK_INT = 329,
+     TK_LONG = 330,
+     TK_FLOAT = 331,
+     TK_DOUBLE = 332,
+     TK_CHAR = 333,
+     TK_WCHAR_T = 334,
+     TK_VOID = 335,
+     TK_PYOBJECT = 336,
+     TK_PYTUPLE = 337,
+     TK_PYLIST = 338,
+     TK_PYDICT = 339,
+     TK_PYCALLABLE = 340,
+     TK_PYSLICE = 341,
+     TK_PYTYPE = 342,
+     TK_PYBUFFER = 343,
+     TK_VIRTUAL = 344,
+     TK_ENUM = 345,
+     TK_SIGNED = 346,
+     TK_UNSIGNED = 347,
+     TK_SCOPE = 348,
+     TK_LOGICAL_OR = 349,
+     TK_CONST = 350,
+     TK_STATIC = 351,
+     TK_SIPSIGNAL = 352,
+     TK_SIPSLOT = 353,
+     TK_SIPANYSLOT = 354,
+     TK_SIPRXCON = 355,
+     TK_SIPRXDIS = 356,
+     TK_SIPSLOTCON = 357,
+     TK_SIPSLOTDIS = 358,
+     TK_SIPSSIZET = 359,
+     TK_NUMBER_VALUE = 360,
+     TK_REAL_VALUE = 361,
+     TK_TYPEDEF = 362,
+     TK_NAMESPACE = 363,
+     TK_TIMELINE = 364,
+     TK_PLATFORMS = 365,
+     TK_FEATURE = 366,
+     TK_LICENSE = 367,
+     TK_QCHAR_VALUE = 368,
+     TK_TRUE_VALUE = 369,
+     TK_FALSE_VALUE = 370,
+     TK_NULL_VALUE = 371,
+     TK_OPERATOR = 372,
+     TK_THROW = 373,
+     TK_QOBJECT = 374,
+     TK_EXCEPTION = 375,
+     TK_RAISECODE = 376,
+     TK_VIRTERRORCODE = 377,
+     TK_EXPLICIT = 378,
+     TK_TEMPLATE = 379,
+     TK_FINAL = 380,
+     TK_ELLIPSIS = 381,
+     TK_DEFMETATYPE = 382,
+     TK_DEFSUPERTYPE = 383,
+     TK_PROPERTY = 384,
+     TK_HIDE_NS = 385,
+     TK_FORMAT = 386,
+     TK_GET = 387,
+     TK_ID = 388,
+     TK_KWARGS = 389,
+     TK_LANGUAGE = 390,
+     TK_LICENSEE = 391,
+     TK_NAME = 392,
+     TK_OPTIONAL = 393,
+     TK_ORDER = 394,
+     TK_REMOVELEADING = 395,
+     TK_SET = 396,
+     TK_SIGNATURE = 397,
+     TK_TIMESTAMP = 398,
+     TK_TYPE = 399,
+     TK_USEARGNAMES = 400,
+     TK_USELIMITEDAPI = 401,
+     TK_ALLRAISEPYEXC = 402,
+     TK_CALLSUPERINIT = 403,
+     TK_DEFERRORHANDLER = 404,
+     TK_VERSION = 405
    };
 #endif
 /* Tokens.  */
@@ -231,112 +235,116 @@
 #define TK_PICKLECODE 300
 #define TK_VIRTUALCALLCODE 301
 #define TK_METHODCODE 302
-#define TK_INSTANCECODE 303
-#define TK_FROMTYPE 304
-#define TK_TOTYPE 305
-#define TK_TOSUBCLASS 306
-#define TK_INCLUDE 307
-#define TK_OPTINCLUDE 308
-#define TK_IMPORT 309
-#define TK_EXPHEADERCODE 310
-#define TK_MODHEADERCODE 311
-#define TK_TYPEHEADERCODE 312
-#define TK_MODULE 313
-#define TK_CMODULE 314
-#define TK_CONSMODULE 315
-#define TK_COMPOMODULE 316
-#define TK_CLASS 317
-#define TK_STRUCT 318
-#define TK_PUBLIC 319
-#define TK_PROTECTED 320
-#define TK_PRIVATE 321
-#define TK_SIGNALS 322
-#define TK_SIGNAL_METHOD 323
-#define TK_SLOTS 324
-#define TK_SLOT_METHOD 325
-#define TK_BOOL 326
-#define TK_SHORT 327
-#define TK_INT 328
-#define TK_LONG 329
-#define TK_FLOAT 330
-#define TK_DOUBLE 331
-#define TK_CHAR 332
-#define TK_WCHAR_T 333
-#define TK_VOID 334
-#define TK_PYOBJECT 335
-#define TK_PYTUPLE 336
-#define TK_PYLIST 337
-#define TK_PYDICT 338
-#define TK_PYCALLABLE 339
-#define TK_PYSLICE 340
-#define TK_PYTYPE 341
-#define TK_PYBUFFER 342
-#define TK_VIRTUAL 343
-#define TK_ENUM 344
-#define TK_SIGNED 345
-#define TK_UNSIGNED 346
-#define TK_SCOPE 347
-#define TK_LOGICAL_OR 348
-#define TK_CONST 349
-#define TK_STATIC 350
-#define TK_SIPSIGNAL 351
-#define TK_SIPSLOT 352
-#define TK_SIPANYSLOT 353
-#define TK_SIPRXCON 354
-#define TK_SIPRXDIS 355
-#define TK_SIPSLOTCON 356
-#define TK_SIPSLOTDIS 357
-#define TK_SIPSSIZET 358
-#define TK_NUMBER_VALUE 359
-#define TK_REAL_VALUE 360
-#define TK_TYPEDEF 361
-#define TK_NAMESPACE 362
-#define TK_TIMELINE 363
-#define TK_PLATFORMS 364
-#define TK_FEATURE 365
-#define TK_LICENSE 366
-#define TK_QCHAR_VALUE 367
-#define TK_TRUE_VALUE 368
-#define TK_FALSE_VALUE 369
-#define TK_NULL_VALUE 370
-#define TK_OPERATOR 371
-#define TK_THROW 372
-#define TK_QOBJECT 373
-#define TK_EXCEPTION 374
-#define TK_RAISECODE 375
-#define TK_VIRTERRORCODE 376
-#define TK_EXPLICIT 377
-#define TK_TEMPLATE 378
-#define TK_ELLIPSIS 379
-#define TK_DEFMETATYPE 380
-#define TK_DEFSUPERTYPE 381
-#define TK_PROPERTY 382
-#define TK_FORMAT 383
-#define TK_GET 384
-#define TK_ID 385
-#define TK_KWARGS 386
-#define TK_LANGUAGE 387
-#define TK_LICENSEE 388
-#define TK_NAME 389
-#define TK_OPTIONAL 390
-#define TK_ORDER 391
-#define TK_REMOVELEADING 392
-#define TK_SET 393
-#define TK_SIGNATURE 394
-#define TK_TIMESTAMP 395
-#define TK_TYPE 396
-#define TK_USEARGNAMES 397
-#define TK_ALLRAISEPYEXC 398
-#define TK_CALLSUPERINIT 399
-#define TK_DEFERRORHANDLER 400
-#define TK_VERSION 401
+#define TK_PREMETHODCODE 303
+#define TK_INSTANCECODE 304
+#define TK_FROMTYPE 305
+#define TK_TOTYPE 306
+#define TK_TOSUBCLASS 307
+#define TK_INCLUDE 308
+#define TK_OPTINCLUDE 309
+#define TK_IMPORT 310
+#define TK_EXPHEADERCODE 311
+#define TK_MODHEADERCODE 312
+#define TK_TYPEHEADERCODE 313
+#define TK_MODULE 314
+#define TK_CMODULE 315
+#define TK_CONSMODULE 316
+#define TK_COMPOMODULE 317
+#define TK_CLASS 318
+#define TK_STRUCT 319
+#define TK_PUBLIC 320
+#define TK_PROTECTED 321
+#define TK_PRIVATE 322
+#define TK_SIGNALS 323
+#define TK_SIGNAL_METHOD 324
+#define TK_SLOTS 325
+#define TK_SLOT_METHOD 326
+#define TK_BOOL 327
+#define TK_SHORT 328
+#define TK_INT 329
+#define TK_LONG 330
+#define TK_FLOAT 331
+#define TK_DOUBLE 332
+#define TK_CHAR 333
+#define TK_WCHAR_T 334
+#define TK_VOID 335
+#define TK_PYOBJECT 336
+#define TK_PYTUPLE 337
+#define TK_PYLIST 338
+#define TK_PYDICT 339
+#define TK_PYCALLABLE 340
+#define TK_PYSLICE 341
+#define TK_PYTYPE 342
+#define TK_PYBUFFER 343
+#define TK_VIRTUAL 344
+#define TK_ENUM 345
+#define TK_SIGNED 346
+#define TK_UNSIGNED 347
+#define TK_SCOPE 348
+#define TK_LOGICAL_OR 349
+#define TK_CONST 350
+#define TK_STATIC 351
+#define TK_SIPSIGNAL 352
+#define TK_SIPSLOT 353
+#define TK_SIPANYSLOT 354
+#define TK_SIPRXCON 355
+#define TK_SIPRXDIS 356
+#define TK_SIPSLOTCON 357
+#define TK_SIPSLOTDIS 358
+#define TK_SIPSSIZET 359
+#define TK_NUMBER_VALUE 360
+#define TK_REAL_VALUE 361
+#define TK_TYPEDEF 362
+#define TK_NAMESPACE 363
+#define TK_TIMELINE 364
+#define TK_PLATFORMS 365
+#define TK_FEATURE 366
+#define TK_LICENSE 367
+#define TK_QCHAR_VALUE 368
+#define TK_TRUE_VALUE 369
+#define TK_FALSE_VALUE 370
+#define TK_NULL_VALUE 371
+#define TK_OPERATOR 372
+#define TK_THROW 373
+#define TK_QOBJECT 374
+#define TK_EXCEPTION 375
+#define TK_RAISECODE 376
+#define TK_VIRTERRORCODE 377
+#define TK_EXPLICIT 378
+#define TK_TEMPLATE 379
+#define TK_FINAL 380
+#define TK_ELLIPSIS 381
+#define TK_DEFMETATYPE 382
+#define TK_DEFSUPERTYPE 383
+#define TK_PROPERTY 384
+#define TK_HIDE_NS 385
+#define TK_FORMAT 386
+#define TK_GET 387
+#define TK_ID 388
+#define TK_KWARGS 389
+#define TK_LANGUAGE 390
+#define TK_LICENSEE 391
+#define TK_NAME 392
+#define TK_OPTIONAL 393
+#define TK_ORDER 394
+#define TK_REMOVELEADING 395
+#define TK_SET 396
+#define TK_SIGNATURE 397
+#define TK_TIMESTAMP 398
+#define TK_TYPE 399
+#define TK_USEARGNAMES 400
+#define TK_USELIMITEDAPI 401
+#define TK_ALLRAISEPYEXC 402
+#define TK_CALLSUPERINIT 403
+#define TK_DEFERRORHANDLER 404
+#define TK_VERSION 405
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 193 "/Users/phil/hg/sip/sip-4.18/sipgen/metasrc/parser.y"
+#line 195 "sip-4.19.1/sipgen/metasrc/parser.y"
 {
     char            qchar;
     char            *text;
@@ -364,6 +372,7 @@ typedef union YYSTYPE
     defEncodingCfg  defencoding;
     defMetatypeCfg  defmetatype;
     defSupertypeCfg defsupertype;
+    hiddenNsCfg     hiddenns;
     exceptionCfg    exception;
     docstringCfg    docstring;
     extractCfg      extract;
@@ -379,7 +388,7 @@ typedef union YYSTYPE
     int             token;
 }
 /* Line 1529 of yacc.c.  */
-#line 383 "/Users/phil/hg/sip/sip-4.18/sipgen/parser.h"
+#line 392 "sip-4.19.1/sipgen/parser.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
