@@ -1,7 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2016 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -27,8 +27,8 @@
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x041007
-#define SIP_VERSION_STR     "4.16.7"
+#define SIP_VERSION         0x041301
+#define SIP_VERSION_STR     "4.19.1"
 
 
 #ifdef TRUE
@@ -43,6 +43,23 @@
 #define FALSE   0
 
 
+/* Some convenient compiler extensions. */
+
+#if defined(__GNUC__)
+#define SIP_NORETURN    __attribute__((__noreturn__))
+#define SIP_UNUSED      __attribute__((__unused__))
+#elif defined(_MSC_VER)
+#define SIP_NORETURN    __declspec(noreturn)
+#endif
+
+#if !defined(SIP_NORETURN)
+#define SIP_NORETURN
+#endif
+#if !defined(SIP_UNUSED)
+#define SIP_UNUSED
+#endif
+
+
 #define DEFAULT_OFILE_EXT   ".o"    /* Default object file extension. */
 
 #define MAX_NR_ARGS         20      /* Max. nr. args. to a function or template. */
@@ -54,6 +71,9 @@
 #define classBaseName(cd)   scopedNameTail((cd)->iff->fqcname)
 #define classFQCName(cd)    ((cd)->iff->fqcname)
 
+/* Return the Python scope corresponding to a C/C++ scope. */
+#define pyScope(c)          ((c) != NULL && isHiddenNamespace(c) ? NULL : (c))
+
 
 /* Handle module flags. */
 
@@ -62,11 +82,12 @@
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
 #define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 #define MOD_USE_ARG_NAMES       0x0010  /* Use real argument names. */
-#define MOD_ALL_RAISE_PY_EXC    0x0020  /* All callable raise a Python exception. */
-#define MOD_SUPER_INIT_NO       0x0040  /* Don't call super().__init__(). */
-#define MOD_SUPER_INIT_YES      0x0080  /* Call super().__init__(). */
+#define MOD_USE_LIMITED_API     0x0020  /* Use the limited API. */
+#define MOD_ALL_RAISE_PY_EXC    0x0040  /* All callable raise a Python exception. */
+#define MOD_SUPER_INIT_NO       0x0080  /* Don't call super().__init__(). */
+#define MOD_SUPER_INIT_YES      0x0100  /* Call super().__init__(). */
 #define MOD_SUPER_INIT_UNDEF    0x0000  /* Calling super().__init__() is undefined. */
-#define MOD_SUPER_INIT_MASK     0x00c0  /* The mask for the above flags. */
+#define MOD_SUPER_INIT_MASK     0x0180  /* The mask for the above flags. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
@@ -79,6 +100,8 @@
 #define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
 #define setUseArgNames(m)   ((m)->modflags |= MOD_USE_ARG_NAMES)
 #define useArgNames(m)      ((m)->modflags & MOD_USE_ARG_NAMES)
+#define setUseLimitedAPI(m) ((m)->modflags |= MOD_USE_LIMITED_API)
+#define useLimitedAPI(m)    ((m)->modflags & MOD_USE_LIMITED_API)
 #define setAllRaisePyException(m)   ((m)->modflags |= MOD_ALL_RAISE_PY_EXC)
 #define allRaisePyException(m)  ((m)->modflags & MOD_ALL_RAISE_PY_EXC)
 #define setCallSuperInitNo(m)   ((m)->modflags = ((m)->modflags & ~MOD_SUPER_INIT_MASK) | MOD_SUPER_INIT_NO)
@@ -184,9 +207,13 @@
 
 
 /* Handle the second group of class flags. */
+
 #define CLASS2_TMPL_ARG     0x01        /* The class is a template argument. */
 #define CLASS2_MIXIN        0x02        /* The class is a mixin. */
 #define CLASS2_EXPORT_DERIVED   0x04    /* Export the derived class declaration. */
+#define CLASS2_HIDDEN_NS    0x08        /* The namespace is hidden. */
+#define CLASS2_USE_TMPL_NAME    0x10    /* Use the template name. */
+#define CLASS2_NEEDS_SHADOW 0x20        /* The class needs a shadow class. */
 
 #define isTemplateArg(cd)   ((cd)->classflags2 & CLASS2_TMPL_ARG)
 #define setTemplateArg(cd)  ((cd)->classflags2 |= CLASS2_TMPL_ARG)
@@ -194,6 +221,12 @@
 #define setMixin(cd)        ((cd)->classflags2 |= CLASS2_MIXIN)
 #define isExportDerived(cd) ((cd)->classflags2 & CLASS2_EXPORT_DERIVED)
 #define setExportDerived(cd)    ((cd)->classflags2 |= CLASS2_EXPORT_DERIVED)
+#define isHiddenNamespace(cd)   ((cd)->classflags2 & CLASS2_HIDDEN_NS)
+#define setHiddenNamespace(cd)  ((cd)->classflags2 |= CLASS2_HIDDEN_NS)
+#define useTemplateName(cd) ((cd)->classflags2 & CLASS2_USE_TMPL_NAME)
+#define setUseTemplateName(cd)  ((cd)->classflags2 |= CLASS2_USE_TMPL_NAME)
+#define needsShadow(cd)      ((cd)->classflags & CLASS2_NEEDS_SHADOW)
+#define setNeedsShadow(cd)   ((cd)->classflags |= CLASS2_NEEDS_SHADOW)
 
 
 /* Handle ctor flags.  These are combined with the section flags. */
@@ -255,6 +288,7 @@
 
 #define ENUM_WAS_PROT       0x00000100  /* It was defined as protected. */
 #define ENUM_NO_SCOPE       0x00000200  /* Omit the member scopes. */
+#define ENUM_NEEDS_ENUM     0x00000400  /* The module needs it. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -265,6 +299,8 @@
 #define resetWasProtectedEnum(e)    ((e)->enumflags &= ~ENUM_WAS_PROT)
 #define isNoScope(e)        ((e)->enumflags & ENUM_NO_SCOPE)
 #define setIsNoScope(e)     ((e)->enumflags |= ENUM_NO_SCOPE)
+#define needsEnum(e)        ((e)->enumflags & ENUM_NEEDS_ENUM)
+#define setNeedsEnum(e)     ((e)->enumflags |= ENUM_NEEDS_ENUM)
 
 
 /* Handle hierarchy flags. */
@@ -305,6 +341,8 @@
 #define OVER_IS_DELATTR     0x04000000  /* It is __delattr__. */
 #define OVER_RAISES_PY_EXC  0x08000000  /* It raises a Python exception. */
 #define OVER_NO_ERROR_HANDLER   0x10000000  /* It doesn't use a virtual error handler. */
+#define OVER_ABORT_ON_EXC   0x20000000  /* It aborts on an exception. */
+#define OVER_IS_FINAL       0x40000000  /* It is a final method. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -363,6 +401,10 @@
 #define setRaisesPyException(o) ((o)->overflags |= OVER_RAISES_PY_EXC)
 #define noErrorHandler(o)   ((o)->overflags & OVER_NO_ERROR_HANDLER)
 #define setNoErrorHandler(o)    ((o)->overflags |= OVER_NO_ERROR_HANDLER)
+#define abortOnException(o)     ((o)->overflags & OVER_ABORT_ON_EXC)
+#define setAbortOnException(o)  ((o)->overflags |= OVER_ABORT_ON_EXC)
+#define isFinal(o)          ((o)->overflags & OVER_IS_FINAL)
+#define setIsFinal(o)       ((o)->overflags |= OVER_IS_FINAL)
 
 
 /* Handle variable flags. */
@@ -451,17 +493,13 @@
 
 /* Handle virtual handler flags. */
 
-#define VH_IS_DUPLICATE     0x01    /* It is a duplicate. */
-#define VH_TRANSFERS        0x02    /* It transfers ownership of the result. */
-#define VH_ABORT_ON_EXC     0x04    /* It aborts on an exception. */
+#define VH_TRANSFERS        0x01    /* It transfers ownership of the result. */
+#define VH_ABORT_ON_EXC     0x02    /* It aborts on an exception. */
 
-#define isDuplicateVH(vh)   ((vh)->vhflags & VH_IS_DUPLICATE)
-#define setIsDuplicateVH(vh)    ((vh)->vhflags |= VH_IS_DUPLICATE)
-#define resetIsDuplicateVH(vh)  ((vh)->vhflags &= ~VH_IS_DUPLICATE)
 #define isTransferVH(vh)    ((vh)->vhflags & VH_TRANSFERS)
 #define setIsTransferVH(vh) ((vh)->vhflags |= VH_TRANSFERS)
-#define abortOnException(vh)    ((vh)->vhflags & VH_ABORT_ON_EXC)
-#define setAbortOnException(vh) ((vh)->vhflags |= VH_ABORT_ON_EXC)
+#define abortOnExceptionVH(vh)  ((vh)->vhflags & VH_ABORT_ON_EXC)
+#define setAbortOnExceptionVH(vh)   ((vh)->vhflags |= VH_ABORT_ON_EXC)
 
 
 /* Handle mapped type flags. */
@@ -564,6 +602,11 @@ typedef enum {
     next_slot,
     setattr_slot,
     delattr_slot,       /* This is local to the parser. */
+    matmul_slot,
+    imatmul_slot,
+    await_slot,
+    aiter_slot,
+    anext_slot,
     no_slot
 } slotType;
 
@@ -631,7 +674,6 @@ typedef enum {
 
 
 /* Value types. */
-
 typedef enum {
     qchar_value,
     string_value,
@@ -643,7 +685,6 @@ typedef enum {
 
 
 /* Version types. */
-
 typedef enum {
     time_qualifier,
     platform_qualifier,
@@ -652,7 +693,6 @@ typedef enum {
 
 
 /* Interface file types. */
-
 typedef enum {
     exception_iface,
     mappedtype_iface,
@@ -661,6 +701,24 @@ typedef enum {
 } ifaceFileType;
 
 
+/* Type hint parse status. */
+typedef enum {
+    needs_parsing,
+    being_parsed,
+    parsed
+} typeHintParseStatus;
+
+
+/* Type hint node type. */
+typedef enum {
+    typing_node,
+    class_node,
+    enum_node,
+    brackets_node,
+    other_node
+} typeHintNodeType;
+
+
 /* A location in a .sip source file. */
 typedef struct {
     int linenr;                         /* The line number. */
@@ -669,7 +727,6 @@ typedef struct {
 
 
 /* A software license. */
-
 typedef struct {
     const char *type;                   /* The license type. */
     const char *licensee;               /* The licensee. */
@@ -679,7 +736,6 @@ typedef struct {
 
 
 /* A version qualifier. */
-
 typedef struct _qualDef {
     const char *name;                   /* The qualifier name. */
     qualType qtype;                     /* The qualifier type. */
@@ -692,7 +748,6 @@ typedef struct _qualDef {
 
 
 /* A scoped name. */
-
 typedef struct _scopedNameDef {
     char *name;                         /* The name. */
     struct _scopedNameDef *next;        /* Next in the scope list. */
@@ -700,7 +755,6 @@ typedef struct _scopedNameDef {
 
 
 /* A name. */
-
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
     const char *text;                   /* The text of the name. */
@@ -711,7 +765,6 @@ typedef struct _nameDef {
 
 
 /* A literal code block. */
-
 typedef struct _codeBlock {
     char *frag;                         /* The code itself. */
     const char *filename;               /* The original file. */
@@ -720,7 +773,6 @@ typedef struct _codeBlock {
 
 
 /* A list of literal code blocks. */
-
 typedef struct _codeBlockList {
     codeBlock *block;                   /* The code block. */
     struct _codeBlockList *next;        /* The next in the list. */
@@ -728,7 +780,6 @@ typedef struct _codeBlockList {
 
 
 /* The arguments to a throw specifier. */
-
 typedef struct _throwArgs {
     int nrArgs;                         /* The number of arguments. */
     struct _exceptionDef *args[MAX_NR_ARGS];    /* The arguments. */
@@ -736,9 +787,9 @@ typedef struct _throwArgs {
 
 
 /* An exception. */
-
 typedef struct _exceptionDef {
     int exceptionnr;                    /* The exception number. */
+    int needed;                         /* The module needs it. */
     struct _ifaceFileDef *iff;          /* The interface file. */
     const char *pyname;                 /* The exception Python name. */
     struct _classDef *cd;               /* The exception class. */
@@ -750,7 +801,6 @@ typedef struct _exceptionDef {
 
 
 /* A value. */
-
 typedef struct _valueDef {
     valueType vtype;                    /* The type. */
     char vunop;                         /* Any unary operator. */
@@ -769,16 +819,17 @@ typedef struct _valueDef {
 
 
 /* A member function argument (or result). */
-
 typedef struct {
     argType atype;                      /* The type. */
     nameDef *name;                      /* The name. */
     const char *doctype;                /* The documented type. */
+    struct _typeHintDef *typehint_in;   /* The PEP 484 input type hint. */
+    struct _typeHintDef *typehint_out;  /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     int argflags;                       /* The argument flags. */
     int nrderefs;                       /* Nr. of dereferences. */
     int derefs[MAX_NR_DEREFS];          /* The const for each dereference. */
     valueDef *defval;                   /* The default value. */
-    const char *docval;                 /* The documented value. */
     int key;                            /* The optional /KeepReference/ key. */
     struct _typedefDef *original_type;  /* The original type if typedef'd. */
     union {
@@ -829,16 +880,35 @@ typedef struct _virtErrorHandler {
 } virtErrorHandler;
 
 
+/* A parsed PEP 484 compliant type hint. */
+typedef struct _typeHintDef {
+    typeHintParseStatus status;         /* The state of the type hint parse. */
+    char *raw_hint;                     /* The raw hint. */
+    struct _typeHintNodeDef *root;      /* The root of parsed nodes. */
+} typeHintDef;
+
+
+/* A node of a parsed type hint. */
+typedef struct _typeHintNodeDef {
+    typeHintNodeType type;              /* The type of the node. */
+    union {
+        const char *name;               /* For typing objects and others. */
+        struct _classDef *cd;           /* For class nodes. */
+        struct _enumDef *ed;            /* For enum nodes. */
+    } u;
+    struct _typeHintNodeDef *children;  /* The list of children. */
+    struct _typeHintNodeDef *next;      /* The next sibling. */
+} typeHintNodeDef;
+
+
 /* A module definition. */
 typedef struct _moduleDef {
     nameDef *fullname;                  /* The full module name. */
     const char *name;                   /* The module base name. */
-    int version;                        /* The module version. */
     apiVersionRangeDef *api_versions;   /* The defined APIs. */
     apiVersionRangeDef *api_ranges;     /* The list of API version ranges. */
     int modflags;                       /* The module flags. */
-    KwArgs kwargs;                      /* The styleof keyword argument support. */
-    int qobjclass;                      /* QObject class, -1 if none. */
+    KwArgs kwargs;                      /* The style of keyword argument support. */
     struct _memberDef *othfuncs;        /* List of other functions. */
     struct _overDef *overs;             /* Global overloads. */
     Format defdocstring;                /* The default docstring format. */
@@ -855,19 +925,18 @@ typedef struct _moduleDef {
     codeBlockList *unitcode;            /* Compilation unit code. */
     codeBlockList *unitpostinccode;     /* Compilation unit post-include code. */
     codeBlockList *docstring;           /* The docstring. */
+    codeBlockList *typehintcode;        /* Type hint code. */
     const char *virt_error_handler;     /* The virtual error handler. */
     int parts;                          /* The number of parts generated. */
     const char *file;                   /* The filename. */
     qualDef *qualifiers;                /* The list of qualifiers. */
-    argDef *types;                      /* The array of numbered types. */
-    int nrtypes;                        /* The number of numbered types. */
+    argDef *needed_types;               /* The array of needed types. */
+    int nr_needed_types;                /* The number of needed types. */
     int nrtimelines;                    /* The nr. of timelines. */
     int nrexceptions;                   /* The nr. of exceptions. */
     int nrtypedefs;                     /* The nr. of typedefs. */
-    int nrvirthandlers;                 /* The nr. of virtual handlers. */
     int nrvirterrorhandlers;            /* The nr. of virtual error handlers. */
     int next_key;                       /* The next key to allocate. */
-    struct _virtHandlerDef *virthandlers;   /* The virtual handlers. */
     licenseDef *license;                /* The software license. */
     struct _classDef *proxies;          /* The list of proxy classes. */
     struct _moduleDef *container;       /* The container module, if any. */
@@ -887,9 +956,9 @@ typedef struct _moduleListDef {
 
 
 /* An interface file definition. */
-
 typedef struct _ifaceFileDef {
     nameDef *name;                      /* The name. */
+    int needed;                         /* The main module needs it. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
     struct _ifaceFileDef *first_alt;    /* The first alternate API. */
     struct _ifaceFileDef *next_alt;     /* The next alternate API. */
@@ -913,13 +982,15 @@ typedef struct _ifaceFileList {
 
 
 /* A mapped type. */
-
 typedef struct _mappedTypeDef {
     int mtflags;                        /* The mapped type flags. */
     argDef type;                        /* The type being mapped. */
     nameDef *pyname;                    /* The Python name. */
     nameDef *cname;                     /* The C/C++ name. */
     const char *doctype;                /* The documented type. */
+    typeHintDef *typehint_in;           /* The PEP 484 input type hint. */
+    typeHintDef *typehint_out;          /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _memberDef *members;         /* The static member functions. */
     struct _overDef *overs;             /* The static overloads. */
@@ -927,12 +998,12 @@ typedef struct _mappedTypeDef {
     codeBlockList *typecode;            /* Type code. */
     codeBlockList *convfromcode;        /* Convert from C++ code. */
     codeBlockList *convtocode;          /* Convert to C++ code. */
+    struct _mappedTypeDef *real;        /* The original definition. */
     struct _mappedTypeDef *next;        /* Next in the list. */
 } mappedTypeDef;
 
 
 /* A function signature. */
-
 typedef struct _signatureDef {
     argDef result;                      /* The result. */
     int nrArgs;                         /* The number of arguments. */
@@ -941,7 +1012,6 @@ typedef struct _signatureDef {
 
 
 /* A list of function signatures. */
-
 typedef struct _signatureList {
     struct _signatureDef *sd;           /* The signature. */
     struct _signatureList *next;        /* Next in the list. */
@@ -949,7 +1019,6 @@ typedef struct _signatureList {
 
 
 /* A template type. */
-
 typedef struct _templateDef {
     scopedNameDef *fqname;              /* The name. */
     signatureDef types;                 /* The types. */
@@ -957,20 +1026,18 @@ typedef struct _templateDef {
 
 
 /* A list of virtual handlers. */
-
 typedef struct _virtHandlerDef {
     int virthandlernr;                  /* The nr. of the virtual handler. */
     int vhflags;                        /* The virtual handler flags. */
     signatureDef *pysig;                /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature. */
-    struct _moduleDef *module;          /* The defining module. */
     codeBlockList *virtcode;            /* Virtual handler code. */
+    virtErrorHandler *veh;              /* The virtual error handler. */
     struct _virtHandlerDef *next;       /* Next in the list. */
 } virtHandlerDef;
 
 
 /* A typedef definition. */
-
 typedef struct _typedefDef {
     int tdflags;                        /* The typedef flags. */
     scopedNameDef *fqname;              /* The fully qualified name. */
@@ -982,10 +1049,10 @@ typedef struct _typedefDef {
 
 
 /* A variable definition. */
-
 typedef struct _varDef {
-    nameDef *pyname;                    /* The variable Python name. */
     scopedNameDef *fqcname;             /* The fully qualified C/C++ name. */
+    nameDef *pyname;                    /* The variable Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     struct _classDef *ecd;              /* The enclosing class. */
     moduleDef *module;                  /* The owning module. */
     int varflags;                       /* The variable flags. */
@@ -998,7 +1065,6 @@ typedef struct _varDef {
 
 
 /* A property definition. */
-
 typedef struct _propertyDef {
     nameDef *name;                      /* The property name. */
     const char *get;                    /* The name of the getter method. */
@@ -1009,11 +1075,11 @@ typedef struct _propertyDef {
 
 
 /* An overloaded member function definition. */
-
 typedef struct _overDef {
     sourceLocation sloc;                /* The source location. */
     char *cppname;                      /* The C++ name. */
     int overflags;                      /* The overload flags. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     int pyqt_signal_hack;               /* The PyQt signal hack. */
     KwArgs kwargs;                      /* The keyword argument support. */
     struct _memberDef *common;          /* Common parts. */
@@ -1022,8 +1088,9 @@ typedef struct _overDef {
     signatureDef *cppsig;               /* The C++ signature. */
     throwArgs *exceptions;              /* The exceptions. */
     codeBlockList *methodcode;          /* Method code. */
-    codeBlockList *virtcallcode;        /* Virtual call  code. */
-    virtHandlerDef *virthandler;        /* The virtual handler. */
+    codeBlockList *premethodcode;       /* Code to insert before the method code. */
+    codeBlockList *virtcallcode;        /* Virtual call code. */
+    codeBlockList *virtcode;            /* Virtual handler code. */
     char *prehook;                      /* The pre-hook name. */
     char *posthook;                     /* The post-hook name. */
     const char *virt_error_handler;     /* The virtual error handler. */
@@ -1032,15 +1099,16 @@ typedef struct _overDef {
 
 
 /* An overloaded constructor definition. */
-
 typedef struct _ctorDef {
     int ctorflags;                      /* The ctor flags. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     KwArgs kwargs;                      /* The keyword argument support. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature, NULL if /NoDerived/. */
     throwArgs *exceptions;              /* The exceptions. */
     codeBlockList *methodcode;          /* Method code. */
+    codeBlockList *premethodcode;       /* Code to insert before the method code. */
     char *prehook;                      /* The pre-hook name. */
     char *posthook;                     /* The post-hook name. */
     struct _ctorDef *next;              /* Next in the list. */
@@ -1048,9 +1116,9 @@ typedef struct _ctorDef {
 
 
 /* An enumerated type member definition. */
-
 typedef struct _enumMemberDef {
     nameDef *pyname;                    /* The Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     char *cname;                        /* The C/C++ name. */
     struct _enumDef *ed;                /* The enclosing enum. */
     struct _enumMemberDef *next;        /* Next in the list. */
@@ -1058,12 +1126,12 @@ typedef struct _enumMemberDef {
 
 
 /* An enumerated type definition. */
-
 typedef struct _enumDef {
     int enumflags;                      /* The enum flags. */
-    nameDef *pyname;                    /* The Python name (may be NULL). */
     scopedNameDef *fqcname;             /* The C/C++ name (may be NULL). */
     nameDef *cname;                     /* The C/C++ name (may be NULL). */
+    nameDef *pyname;                    /* The Python name (may be NULL). */
+    int no_typehint;                    /* The type hint will be suppressed. */
     struct _enumDef *first_alt;         /* The first alternate API. */
     struct _enumDef *next_alt;          /* The next alternate API. */
     int enumnr;                         /* The enum number. */
@@ -1079,7 +1147,6 @@ typedef struct _enumDef {
 
 
 /* An member function definition. */
-
 typedef struct _memberDef {
     nameDef *pyname;                    /* The Python name. */
     int memberflags;                    /* The member flags. */
@@ -1087,12 +1154,12 @@ typedef struct _memberDef {
     slotType slot;                      /* The slot type. */
     moduleDef *module;                  /* The owning module. */
     codeBlockList *docstring;           /* The overloads docstrings. */
+    struct _ifaceFileDef *ns_scope;     /* The scope if it has been moved. */
     struct _memberDef *next;            /* Next in the list. */
 } memberDef;
 
 
 /* A list of visible member functions. */
-
 typedef struct _visibleList {
     memberDef *m;                       /* The member definition. */
     struct _classDef *cd;               /* The class. */
@@ -1101,7 +1168,6 @@ typedef struct _visibleList {
 
 
 /* An entry in a linked class list. */
-
 typedef struct _classList {
     struct _classDef *cd;               /* The class itself. */
     struct _classList *next;            /* Next in the list. */
@@ -1109,15 +1175,14 @@ typedef struct _classList {
 
 
 /* A virtual overload definition. */
-
 typedef struct _virtOverDef {
-    overDef o;                          /* The overload. */
+    overDef *od;                        /* The overload. */
+    virtHandlerDef *virthandler;        /* The virtual handler. */
     struct _virtOverDef *next;          /* Next in the list. */
 } virtOverDef;
 
 
 /* A class that appears in a class's hierarchy. */
-
 typedef struct _mroDef {
     struct _classDef *cd;               /* The class. */
     int mroflags;                       /* The hierarchy flags. */
@@ -1126,13 +1191,13 @@ typedef struct _mroDef {
 
 
 /* A class definition. */
-
 typedef struct _classDef {
     unsigned classflags;                /* The class flags. */
     unsigned classflags2;               /* The class flags, part 2. */
     int pyqt_flags;                     /* The PyQt specific flags. */
     const char *pyqt_interface;         /* The Qt interface name. */
     nameDef *pyname;                    /* The Python name. */
+    int no_typehint;                    /* The type hint will be suppressed. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
@@ -1168,14 +1233,17 @@ typedef struct _classDef {
     codeBlockList *charbufcode;         /* Character buffer code (Python v2). */
     codeBlockList *picklecode;          /* Pickle code. */
     codeBlockList *finalcode;           /* Finalisation code. */
+    codeBlockList *typehintcode;        /* Type hint code. */
     propertyDef *properties;            /* The properties. */
     const char *virt_error_handler;     /* The virtual error handler. */
+    typeHintDef *typehint_in;           /* The PEP 484 input type hint. */
+    typeHintDef *typehint_out;          /* The PEP 484 output type hint. */
+    const char *typehint_value;         /* The type hint value. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
 
 
 /* A class template definition. */
-
 typedef struct _classTmplDef {
     signatureDef sig;                   /* The template arguments. */
     classDef *cd;                       /* The class itself. */
@@ -1184,7 +1252,6 @@ typedef struct _classTmplDef {
 
 
 /* A mapped type template definition. */
-
 typedef struct _mappedTypeTmplDef {
     signatureDef sig;                   /* The template arguments. */
     mappedTypeDef *mt;                  /* The mapped type itself. */
@@ -1193,7 +1260,6 @@ typedef struct _mappedTypeTmplDef {
 
 
 /* The extracts for an identifier. */
-
 typedef struct _extractDef {
     const char *id;                     /* The identifier. */
     struct _extractPartDef *parts;      /* The ordered list of parts. */
@@ -1202,7 +1268,6 @@ typedef struct _extractDef {
 
 
 /* Part of an extract for an identifier. */
-
 typedef struct _extractPartDef {
     int order;                          /* The order of the part. */
     codeBlock *part;                    /* The part itself. */
@@ -1211,7 +1276,6 @@ typedef struct _extractPartDef {
 
 
 /* A rule for automatic Python naming. */
-
 typedef struct _autoPyNameDef {
     const char *remove_leading;         /* Leading string to remove. */
     struct _autoPyNameDef *next;        /* The next in the list. */
@@ -1219,7 +1283,6 @@ typedef struct _autoPyNameDef {
 
 
 /* The parse tree corresponding to the specification file. */
-
 typedef struct {
     moduleDef *module;                  /* The module being generated. */
     moduleDef *modules;                 /* The list of modules. */
@@ -1233,9 +1296,13 @@ typedef struct {
     enumDef *enums;                     /* List of enums. */
     varDef *vars;                       /* List of variables. */
     typedefDef *typedefs;               /* List of typedefs. */
+    int nrvirthandlers;                 /* The number of virtual handlers. */
+    virtHandlerDef *virthandlers;       /* The virtual handlers. */
     virtErrorHandler *errorhandlers;    /* The list of virtual error handlers. */
     codeBlockList *exphdrcode;          /* Exported header code. */
+    codeBlockList *exptypehintcode;     /* Exported type hint code. */
     codeBlockList *docs;                /* Documentation. */
+    classDef *qobject_cd;               /* QObject class, NULL if none. */
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
@@ -1244,7 +1311,6 @@ typedef struct {
 
 
 /* A list of strings. */
-
 typedef struct _stringList {
     const char *s;                      /* The string. */
     struct _stringList *next;           /* The next in the list. */
@@ -1252,7 +1318,6 @@ typedef struct _stringList {
 
 
 /* File specific context information for the parser. */
-
 typedef struct _parserContext {
     const char *filename;               /* The %Import or %Include filename. */
     int ifdepth;                        /* The depth of nested if's. */
@@ -1275,11 +1340,13 @@ void generateExtracts(sipSpec *pt, const stringList *extracts);
 void addExtractPart(sipSpec *pt, const char *id, int order, codeBlock *part);
 void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile);
 void generateXML(sipSpec *pt, moduleDef *mod, const char *xmlFile);
+void generateTypeHints(sipSpec *pt, moduleDef *mod, const char *pyiFile);
 void generateExpression(valueDef *vd, int in_str, FILE *fp);
 void warning(Warning w, const char *fmt, ...);
 void deprecated(const char *msg);
-void fatal(char *,...);
+SIP_NORETURN void fatal(const char *fmt,...);
 void fatalScopedName(scopedNameDef *);
+void fatalStart();
 void getSourceLocation(sourceLocation *slp);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
 void resetLexerState();
@@ -1288,7 +1355,7 @@ void *sipCalloc(size_t nr, size_t n);
 char *sipStrdup(const char *);
 char *concat(const char *, ...);
 void append(char **, const char *);
-void addToUsedList(ifaceFileList **, ifaceFileDef *);
+void appendToIfaceFileList(ifaceFileList **ifflp, ifaceFileDef *iff);
 int selectedQualifier(stringList *needed_qualifiers, qualDef *qd);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
@@ -1303,11 +1370,11 @@ void freeScopedName(scopedNameDef *);
 void appendToClassList(classList **,classDef *);
 void appendCodeBlockList(codeBlockList **headp, codeBlockList *cbl);
 void prcode(FILE *fp, const char *fmt, ...);
+void prCopying(FILE *fp, moduleDef *mod, const char *comment);
 void prOverloadName(FILE *fp, overDef *od);
 void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
+void prDefaultValue(argDef *ad, int in_str, FILE *fp);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
-int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str, int is_signal);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isZeroArgSlot(memberDef *md);
 int isIntReturnSlot(memberDef *md);
@@ -1325,14 +1392,27 @@ codeBlockList *templateCode(sipSpec *pt, ifaceFileList **used,
 ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
         scopedNameDef *fqname, ifaceFileType iftype,
         apiVersionRangeDef *api_range, argDef *ad);
-int pluginPyQt3(sipSpec *pt);
 int pluginPyQt4(sipSpec *pt);
 int pluginPyQt5(sipSpec *pt);
+SIP_NORETURN void yyerror(char *);
 void yywarning(char *);
+int yylex();
 nameDef *cacheName(sipSpec *pt, const char *name);
 scopedNameDef *encodedTemplateName(templateDef *td);
 apiVersionRangeDef *findAPI(sipSpec *pt, const char *name);
 memberDef *findMethod(classDef *cd, const char *name);
+typeHintDef *newTypeHint(char *raw_hint);
+int isPyKeyword(const char *word);
+void getDefaultImplementation(sipSpec *pt, argType atype, classDef **cdp,
+        mappedTypeDef **mtdp);
+char *templateString(const char *src, scopedNameDef *names,
+        scopedNameDef *values);
+int inDefaultAPI(sipSpec *pt, apiVersionRangeDef *range);
+int hasImplicitOverloads(signatureDef *sd);
+void dsCtor(sipSpec *pt, classDef *cd, ctorDef *ct, int sec, FILE *fp);
+void dsOverload(sipSpec *pt, overDef *od, int is_method, int sec, FILE *fp);
+scopedNameDef *getFQCNameOfType(argDef *ad);
+scopedNameDef *removeGlobalScope(scopedNameDef *snd);
 
 
 /* These are only here because bison publically references them. */
@@ -1447,6 +1527,12 @@ typedef struct _featureCfg {
     const char *name;
 } featureCfg;
 
+/* %HiddenNamespace */
+typedef struct _hiddenNsCfg {
+    int token;
+    scopedNameDef *name;
+} hiddenNsCfg;
+
 /* %Import */
 typedef struct _importCfg {
     int token;
@@ -1476,10 +1562,10 @@ typedef struct _moduleCfg {
     KwArgs kwargs;
     const char *name;
     int use_arg_names;
+    int use_limited_api;
     int all_raise_py_exc;
     int call_super_init;
     const char *def_error_handler;
-    int version;
     codeBlock *docstring;
 } moduleCfg;
 
